; Tcl4mIRC support script for version 1.0 
; 
; Written by Loren Segal, clb@soen.ca
;
; This file is not needed to use tcl4mirc.dll but 
; provides a simplified interface to access it.
;
; See USAGE.txt for information on how to use
; the commands defined here, or look at the
; Examples below.

; Convenience methods
alias tcl_dll { return $qt($scriptdirtcl4mirc.dll) }
alias tcl_unload { dll -u $tcl_dll }
alias tcl_str { return $qt($replace($1,\,\\,$,\$)) }
alias tcl { if ($isid) return $dll($tcl_dll,tcl_eval,$1-) | dll $tcl_dll tcl_eval $1- }

; Tcl interpeter bridge methods for embedded scripts
alias tcl_embed { tcl mirc_embed $tcl_str($1) $2- | return $false }
alias has_cached_tcl { return $!tcl_embed($script,$scriptline,-cache) }
alias has_tcl { return $!tcl_embed($script,$scriptline) }

; Helper alias to load Tk and setup the X button to not close the app
alias tk_init { tcl package require Tk | tcl wm protocol . WM_DELETE_WINDOW {wm withdraw .} }

; Standard input/output handling
on *:SIGNAL:TCL_STDOUT:if ($1 != $null) echo -a $1- | else linesep -a
on *:SIGNAL:TCL_STDERR_BEGIN: { linesep -a | set %__tcl_stderr $true }
on *:SIGNAL:TCL_STDERR_END:linesep -a
on *:SIGNAL:TCL_STDERR: { echo $color(info) -a $iif(%__tcl_stderr,tcl: error:) $1- | unset %__tcl_stderr }

; Error callback
on *:SIGNAL:TCL_ERROR:echo $color(info) -ae * Tcl4mIRC: Critical Error: $1- | dll -u $tcl_dll

; Initialization callback
on *:SIGNAL:TCL_LOADED: {
  echo $color(info2) -ae * Loaded Tcl4mIRC version $tcl(package present mirc) $+ . Edit line $scriptline of $qt($remove($script,$mircdir)) to change this message.
  if (!$1) {
    ; Tcl4mIRC is not in supported mode
    echo $color(info) -ae * Tcl is not running in supported mode. Your version of mIRC ( $+ $version $+ ) is not compatible with Tcl4mIRC.
  }
}
on *:LOAD: { echo $color(info2) -ae * Running /tcl_test to see if Tcl works: | tcl_test }

;
; Examples
;

; One-liners

; Classic hello world
alias tcl_hello_world { tcl puts "Hello world" }

; Timer-delays the Tcl way
alias tcl_sleep { tcl after 1000 puts "Done"; vwait end }

; Download a file from a website and print the first line (needs Tcl installed)
alias tcl_get_versions_file {
  tcl package require http
  tcl puts [lindex [split [::http::data [::http::geturl http://www.mirc.co.uk/versions.txt]] \n] 0]
}

; Shows how to pass data to and from Tcl when certain identifiers 
; are not accessible such as $1-
alias tcl_strlen {
  set %data $1-
  tcl puts [string length [mirc {%data}]]
  unset %data
}

; Embedded Tcl

; Test method
alias tcl_test {
  if $($has_tcl,2) {
    mirc linesep -a
    set sorted [lsort {3 2 1}]
    puts "Testing Tcl... here is a sorted array: $sorted"
    if {[mirc \$sorttok(1 6 2, 32)] == {1 2 6}} {
      	puts "Tcl works great!"
    }
    mirc linesep -a
  }
}

; Lists the packages currently loaded in Tcl
alias tcl_list_packages {
  if $($has_tcl,2) {
    set res {}
    foreach i [package names] {
      if {[string length [package provide $i]]} {
        lappend res $i
      }
    }
    # Bring information back to mIRC in a var rather
    # than using the mirc proc to /echo the results
    mirc set %packages $res
  }
  echo -a Tcl Packages: %packages
  unset %packages
}

; Here is a Tk example of a game of checkers
; Taken from http://wiki.tcl.tk/884
alias tcl_tk_checkers {
  tk_init
  
  if $($has_tcl,2) {
    set dx 50
    set dy 50
    #set colors {black white red blue}
    set colors {white black red blue}

    pack [canvas .c -width [expr $dx*8] -height [expr $dy*8]] -fill both -expand 1

    .c bind all <1> {set c(X) [.c canvasx %x]; set c(Y) [.c canvasy %y]}
    .c bind mv <B1-Motion> {mv %x %y}
    .c bind mv <ButtonRelease-1> {drop [.c find withtag current]}
    proc drop {id} {
      global c dx dy
      set cx0 [expr {int($c(X) / $dx) * $dx + 2}]
      set cy0 [expr {int($c(Y) / $dy) * $dy + 2}]
      set cx1 [expr {$cx0 + $dx - 3}]
      set cy1 [expr {$cy0 + $dy - 3}]
      .c coords $id $cx0 $cy0 $cx1 $cy1
    }

    proc mv {x y} {
      global c
      set x  [.c canvasx $x]
      set y  [.c canvasy $y]
      set id [.c find withtag current]
      .c move $id [expr {$x-$c(X)}] [expr {$y-$c(Y)}]
      .c raise $id
      set c(X) $x; set c(Y) $y
    }

    set color 0
    proc makepiece {x y color args} {
      global dx dy
      .c create oval [expr {$x+2}] [expr {$y+2}] [expr {$x+$dx-3}] [expr {$y+$dy-3}] \
      -fill $color -tags [concat $args mv]
    }
    proc makecheck {x y color args} {
      global dx dy
      .c create rectangle $x $y [expr {$x+$dx}] [expr {$y+$dy}] \
      -fill $color -tags $args
    }
    for {set i 0; set y 0} {$i<8} {incr i; incr y $dy} {
      for {set j 0; set x 0} {$j<8} {incr j; incr x $dx} {
        makecheck $x $y [lindex $colors $color]
        if {$i<3 && $color} {
          makepiece $x $y [lindex $colors 2] player2
        }
        if {$i>4 && $color} {
          makepiece $x $y [lindex $colors 3] player1
        }
        set color [expr {1-$color}]
      }
      set color [expr {1-$color}]
    }
  }
}

; Here is an example of an mIRC getting event specific identifiers
on *:TEXT:*hello*:#tcl-moo-moo: {
  if $($has_tcl,2) {
    set nick [mirc \$nick]
    set chan [mirc \$chan]
    puts "$nick just said hi on $chan"
  }
}

; Simple tclsh implementation
alias tclsh { window -e @tclsh }
on *:INPUT:@tclsh:echo @tclsh % $1- | if ($tcl($1-) != $null) echo @tclsh $v1 | halt
